/* testresource.c generated by valac 0.16.1, the Vala compiler
 * generated from testresource.vala, do not modify */


#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>



extern GMainLoop* loop;
GMainLoop* loop = NULL;

#define DBUS_TEST_BUSNAME "org.freesmartphone.testing"
#define DBUS_TEST_OBJPATH "/org/freesmartphone/Testing"
#define DBUS_TEST_INTERFACE "org.freesmartphone.Testing"
void test_resource_all (void);
void _vala_main (gchar** args, int args_length1);
static void _test_resource_all_gcallback (void);


void test_resource_all (void) {
}


static void _test_resource_all_gcallback (void) {
	test_resource_all ();
}


void _vala_main (gchar** args, int args_length1) {
	g_test_init (&args_length1, &args, NULL);
	g_test_add_func ("/Resource/all", _test_resource_all_gcallback);
	g_test_run ();
}


int main (int argc, char ** argv) {
	g_type_init ();
	_vala_main (argv, argc);
	return 0;
}



