/* resource.c generated by valac 0.16.1, the Vala compiler
 * generated from resource.vala, do not modify */

/*
 * Copyright (C) 2009-2012 Michael 'Mickey' Lauer <mlauer@vanille-media.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

#include <glib.h>
#include <glib-object.h>


#define FSO_FRAMEWORK_TYPE_RESOURCE_STATUS (fso_framework_resource_status_get_type ())

typedef enum  {
	FSO_FRAMEWORK_RESOURCE_STATUS_UNKNOWN,
	FSO_FRAMEWORK_RESOURCE_STATUS_ENABLING,
	FSO_FRAMEWORK_RESOURCE_STATUS_ENABLED,
	FSO_FRAMEWORK_RESOURCE_STATUS_SUSPENDING,
	FSO_FRAMEWORK_RESOURCE_STATUS_SUSPENDED,
	FSO_FRAMEWORK_RESOURCE_STATUS_RESUMING,
	FSO_FRAMEWORK_RESOURCE_STATUS_DISABLING,
	FSO_FRAMEWORK_RESOURCE_STATUS_DISABLED
} FsoFrameworkResourceStatus;



GType fso_framework_resource_status_get_type (void) G_GNUC_CONST;


/**
 * @enum FsoFramework.ResourceStatus
 **/
GType fso_framework_resource_status_get_type (void) {
	static volatile gsize fso_framework_resource_status_type_id__volatile = 0;
	if (g_once_init_enter (&fso_framework_resource_status_type_id__volatile)) {
		static const GEnumValue values[] = {{FSO_FRAMEWORK_RESOURCE_STATUS_UNKNOWN, "FSO_FRAMEWORK_RESOURCE_STATUS_UNKNOWN", "unknown"}, {FSO_FRAMEWORK_RESOURCE_STATUS_ENABLING, "FSO_FRAMEWORK_RESOURCE_STATUS_ENABLING", "enabling"}, {FSO_FRAMEWORK_RESOURCE_STATUS_ENABLED, "FSO_FRAMEWORK_RESOURCE_STATUS_ENABLED", "enabled"}, {FSO_FRAMEWORK_RESOURCE_STATUS_SUSPENDING, "FSO_FRAMEWORK_RESOURCE_STATUS_SUSPENDING", "suspending"}, {FSO_FRAMEWORK_RESOURCE_STATUS_SUSPENDED, "FSO_FRAMEWORK_RESOURCE_STATUS_SUSPENDED", "suspended"}, {FSO_FRAMEWORK_RESOURCE_STATUS_RESUMING, "FSO_FRAMEWORK_RESOURCE_STATUS_RESUMING", "resuming"}, {FSO_FRAMEWORK_RESOURCE_STATUS_DISABLING, "FSO_FRAMEWORK_RESOURCE_STATUS_DISABLING", "disabling"}, {FSO_FRAMEWORK_RESOURCE_STATUS_DISABLED, "FSO_FRAMEWORK_RESOURCE_STATUS_DISABLED", "disabled"}, {0, NULL, NULL}};
		GType fso_framework_resource_status_type_id;
		fso_framework_resource_status_type_id = g_enum_register_static ("FsoFrameworkResourceStatus", values);
		g_once_init_leave (&fso_framework_resource_status_type_id__volatile, fso_framework_resource_status_type_id);
	}
	return fso_framework_resource_status_type_id__volatile;
}



