/* interfaces.c generated by valac 0.16.1, the Vala compiler
 * generated from interfaces.vala, do not modify */

/*
 * Copyright (C) 2009-2012 Michael 'Mickey' Lauer <mlauer@vanille-media.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */
/* NOTE: This file only contains object paths and interface names.*/
/* The actual interfaces are implemented in libfso-glib.*/

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>




#define FSO_FRAMEWORK_ServiceDBusPrefix "org.freesmartphone"
#define FSO_FRAMEWORK_ServicePathPrefix "/org/freesmartphone"
#define FSO_FRAMEWORK_ServiceFacePrefix "org.freesmartphone"
#define FSO_FRAMEWORK_CONTEXT_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".ocontextd"
#define FSO_FRAMEWORK_CONTEXT_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Context"
#define FSO_FRAMEWORK_CONTEXT_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Context"
#define FSO_FRAMEWORK_CONTEXT_ManagerServiceFace FSO_FRAMEWORK_CONTEXT_ServiceFacePrefix ".Manager"
#define FSO_FRAMEWORK_CONTEXT_ManagerServicePath FSO_FRAMEWORK_CONTEXT_ServicePathPrefix "/Manager"
#define FSO_FRAMEWORK_CONTEXT_ClientServiceFace FSO_FRAMEWORK_CONTEXT_ServiceFacePrefix ".Client"
#define FSO_FRAMEWORK_CONTEXT_ClientServicePath FSO_FRAMEWORK_CONTEXT_ServicePathPrefix "/Client"
#define FSO_FRAMEWORK_DATA_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".odatad"
#define FSO_FRAMEWORK_DATA_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Data"
#define FSO_FRAMEWORK_DATA_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Data"
#define FSO_FRAMEWORK_DATA_WorldServiceFace FSO_FRAMEWORK_DATA_ServiceFacePrefix ".World"
#define FSO_FRAMEWORK_DATA_WorldServicePath FSO_FRAMEWORK_DATA_ServicePathPrefix "/World"
#define FSO_FRAMEWORK_DEVICE_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".odeviced"
#define FSO_FRAMEWORK_DEVICE_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Device"
#define FSO_FRAMEWORK_DEVICE_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Device"
#define FSO_FRAMEWORK_DEVICE_AmbientLightServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".AmbientLight"
#define FSO_FRAMEWORK_DEVICE_AmbientLightServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/AmbientLight"
#define FSO_FRAMEWORK_DEVICE_AudioServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".Audio"
#define FSO_FRAMEWORK_DEVICE_AudioServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/Audio"
#define FSO_FRAMEWORK_DEVICE_DisplayServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".Display"
#define FSO_FRAMEWORK_DEVICE_DisplayServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/Display"
#define FSO_FRAMEWORK_DEVICE_IdleNotifierServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".IdleNotifier"
#define FSO_FRAMEWORK_DEVICE_IdleNotifierServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/IdleNotifier"
#define FSO_FRAMEWORK_DEVICE_InfoServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".Info"
#define FSO_FRAMEWORK_DEVICE_InfoServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/Info"
#define FSO_FRAMEWORK_DEVICE_InputServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".Input"
#define FSO_FRAMEWORK_DEVICE_InputServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/Input"
#define FSO_FRAMEWORK_DEVICE_LedServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".LED"
#define FSO_FRAMEWORK_DEVICE_LedServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/LED"
#define FSO_FRAMEWORK_DEVICE_OrientationServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".Orientation"
#define FSO_FRAMEWORK_DEVICE_OrientationServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/Orientation"
#define FSO_FRAMEWORK_DEVICE_PowerControlServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".PowerControl"
#define FSO_FRAMEWORK_DEVICE_PowerControlServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/PowerControl"
#define FSO_FRAMEWORK_DEVICE_PowerSupplyServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".PowerSupply"
#define FSO_FRAMEWORK_DEVICE_PowerSupplyServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/PowerSupply"
#define FSO_FRAMEWORK_DEVICE_ProximityServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".Proximity"
#define FSO_FRAMEWORK_DEVICE_ProximityServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/Proximity"
#define FSO_FRAMEWORK_DEVICE_RtcServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".RTC"
#define FSO_FRAMEWORK_DEVICE_RtcServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/RTC"
#define FSO_FRAMEWORK_DEVICE_VibratorServiceFace FSO_FRAMEWORK_DEVICE_ServiceFacePrefix ".Vibrator"
#define FSO_FRAMEWORK_DEVICE_VibratorServicePath FSO_FRAMEWORK_DEVICE_ServicePathPrefix "/Vibrator"
#define FSO_FRAMEWORK_GPS_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".ogpsd"
#define FSO_FRAMEWORK_GPS_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".GPS"
#define FSO_FRAMEWORK_GPS_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/GPS"
#define FSO_FRAMEWORK_GPS_DeviceServiceFace FSO_FRAMEWORK_GPS_ServiceFacePrefix ".Device"
#define FSO_FRAMEWORK_GPS_DeviceServicePath FSO_FRAMEWORK_GPS_ServicePathPrefix "/Device"
#define FSO_FRAMEWORK_GSM_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".ogsmd"
#define FSO_FRAMEWORK_GSM_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".GSM"
#define FSO_FRAMEWORK_GSM_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/GSM"
#define FSO_FRAMEWORK_GSM_DeviceServiceFace FSO_FRAMEWORK_GSM_ServiceFacePrefix ".Device"
#define FSO_FRAMEWORK_GSM_DeviceServicePath FSO_FRAMEWORK_GSM_ServicePathPrefix "/Device"
#define FSO_FRAMEWORK_NETWORK_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".onetworkd"
#define FSO_FRAMEWORK_NETWORK_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Network"
#define FSO_FRAMEWORK_NETWORK_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Network"
#define FSO_FRAMEWORK_PHONE_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".ophoned"
#define FSO_FRAMEWORK_PHONE_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Phone"
#define FSO_FRAMEWORK_PHONE_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Phone"
#define FSO_FRAMEWORK_PHONE_ManagerServiceFace FSO_FRAMEWORK_PHONE_ServiceFacePrefix ".Manager"
#define FSO_FRAMEWORK_PHONE_ManagerServicePath FSO_FRAMEWORK_PHONE_ServicePathPrefix "/Manager"
#define FSO_FRAMEWORK_PHONE_ClientServiceFace FSO_FRAMEWORK_PHONE_ServiceFacePrefix ".Client"
#define FSO_FRAMEWORK_PHONE_ClientServicePath FSO_FRAMEWORK_PHONE_ServicePathPrefix "/Client"
#define FSO_FRAMEWORK_PREFERENCES_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".opreferencesd"
#define FSO_FRAMEWORK_PREFERENCES_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Preferences"
#define FSO_FRAMEWORK_PREFERENCES_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Preferences"
#define FSO_FRAMEWORK_PIM_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".opimd"
#define FSO_FRAMEWORK_PIM_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".PIM"
#define FSO_FRAMEWORK_PIM_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/PIM"
#define FSO_FRAMEWORK_PIM_MessagesServiceFace FSO_FRAMEWORK_PIM_ServiceFacePrefix ".Messages"
#define FSO_FRAMEWORK_PIM_MessagesServicePath FSO_FRAMEWORK_PIM_ServicePathPrefix "/Messages"
#define FSO_FRAMEWORK_PIM_ContactsServiceFace FSO_FRAMEWORK_PIM_ServiceFacePrefix ".Contacts"
#define FSO_FRAMEWORK_PIM_ContactsServicePath FSO_FRAMEWORK_PIM_ServicePathPrefix "/Contacts"
#define FSO_FRAMEWORK_PIM_ContactServiceFace FSO_FRAMEWORK_PIM_ServiceFacePrefix ".Contact"
#define FSO_FRAMEWORK_PIM_CallsServiceFace FSO_FRAMEWORK_PIM_ServiceFacePrefix ".Calls"
#define FSO_FRAMEWORK_PIM_CallsServicePath FSO_FRAMEWORK_PIM_ServicePathPrefix "/Calls"
#define FSO_FRAMEWORK_PIM_TasksServiceFace FSO_FRAMEWORK_PIM_ServiceFacePrefix ".Tasks"
#define FSO_FRAMEWORK_PIM_TasksServicePath FSO_FRAMEWORK_PIM_ServicePathPrefix "/Tasks"
#define FSO_FRAMEWORK_PIM_NotesServiceFace FSO_FRAMEWORK_PIM_ServiceFacePrefix ".Notes"
#define FSO_FRAMEWORK_PIM_NotesServicePath FSO_FRAMEWORK_PIM_ServicePathPrefix "/Notes"
#define FSO_FRAMEWORK_RESOURCE_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".oresourced"
#define FSO_FRAMEWORK_RESOURCE_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Resource"
#define FSO_FRAMEWORK_RESOURCE_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Resource"
#define FSO_FRAMEWORK_TIME_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".otimed"
#define FSO_FRAMEWORK_TIME_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Time"
#define FSO_FRAMEWORK_TIME_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Time"
#define FSO_FRAMEWORK_TIME_AlarmServiceFace FSO_FRAMEWORK_TIME_ServiceFacePrefix ".Alarm"
#define FSO_FRAMEWORK_TIME_AlarmServicePath FSO_FRAMEWORK_TIME_ServicePathPrefix "/Alarm"
#define FSO_FRAMEWORK_TIME_LocationServiceFace FSO_FRAMEWORK_TIME_ServiceFacePrefix ".Location"
#define FSO_FRAMEWORK_TIME_LocationServicePath FSO_FRAMEWORK_TIME_ServicePathPrefix "/Location"
#define FSO_FRAMEWORK_USAGE_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".ousaged"
#define FSO_FRAMEWORK_USAGE_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Usage"
#define FSO_FRAMEWORK_USAGE_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Usage"
#define FSO_FRAMEWORK_AUDIO_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".oaudiod"
#define FSO_FRAMEWORK_AUDIO_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Audio"
#define FSO_FRAMEWORK_AUDIO_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Audio"
#define FSO_FRAMEWORK_APPLICATION_ServiceDBusName FSO_FRAMEWORK_ServiceDBusPrefix ".oappd"
#define FSO_FRAMEWORK_APPLICATION_ServiceFacePrefix FSO_FRAMEWORK_ServiceFacePrefix ".Application"
#define FSO_FRAMEWORK_APPLICATION_ServicePathPrefix FSO_FRAMEWORK_ServicePathPrefix "/Application"



