/* testcase.c generated by valac 0.16.1, the Vala compiler
 * generated from testcase.vala, do not modify */

/* testcase.vala
 *
 * Copyright (C) 2009-2012 Julien Peeters
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 *  Julien Peeters <contact@julienpeeters.fr>
 *
 * Copied from libgee/tests/testcase.vala.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <stdio.h>
#include <gobject/gvaluecollector.h>


#define FSO_FRAMEWORK_TEST_TYPE_TEST_CASE (fso_framework_test_test_case_get_type ())
#define FSO_FRAMEWORK_TEST_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FSO_FRAMEWORK_TEST_TYPE_TEST_CASE, FsoFrameworkTestTestCase))
#define FSO_FRAMEWORK_TEST_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FSO_FRAMEWORK_TEST_TYPE_TEST_CASE, FsoFrameworkTestTestCaseClass))
#define FSO_FRAMEWORK_TEST_IS_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FSO_FRAMEWORK_TEST_TYPE_TEST_CASE))
#define FSO_FRAMEWORK_TEST_IS_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FSO_FRAMEWORK_TEST_TYPE_TEST_CASE))
#define FSO_FRAMEWORK_TEST_TEST_CASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FSO_FRAMEWORK_TEST_TYPE_TEST_CASE, FsoFrameworkTestTestCaseClass))

typedef struct _FsoFrameworkTestTestCase FsoFrameworkTestTestCase;
typedef struct _FsoFrameworkTestTestCaseClass FsoFrameworkTestTestCaseClass;
typedef struct _FsoFrameworkTestTestCasePrivate FsoFrameworkTestTestCasePrivate;

#define FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR (fso_framework_test_test_case_adaptor_get_type ())
#define FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR, FsoFrameworkTestTestCaseAdaptor))
#define FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR, FsoFrameworkTestTestCaseAdaptorClass))
#define FSO_FRAMEWORK_TEST_TEST_CASE_IS_ADAPTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR))
#define FSO_FRAMEWORK_TEST_TEST_CASE_IS_ADAPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR))
#define FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR, FsoFrameworkTestTestCaseAdaptorClass))

typedef struct _FsoFrameworkTestTestCaseAdaptor FsoFrameworkTestTestCaseAdaptor;
typedef struct _FsoFrameworkTestTestCaseAdaptorClass FsoFrameworkTestTestCaseAdaptorClass;
#define _fso_framework_test_test_case_adaptor_unref0(var) ((var == NULL) ? NULL : (var = (fso_framework_test_test_case_adaptor_unref (var), NULL)))
typedef struct _FsoFrameworkTestTestCaseAdaptorPrivate FsoFrameworkTestTestCaseAdaptorPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
typedef struct _FsoFrameworkTestTestCaseParamSpecAdaptor FsoFrameworkTestTestCaseParamSpecAdaptor;

struct _FsoFrameworkTestTestCase {
	GObject parent_instance;
	FsoFrameworkTestTestCasePrivate * priv;
};

struct _FsoFrameworkTestTestCaseClass {
	GObjectClass parent_class;
	void (*set_up) (FsoFrameworkTestTestCase* self);
	void (*tear_down) (FsoFrameworkTestTestCase* self);
};

struct _FsoFrameworkTestTestCasePrivate {
	GTestSuite* _suite;
	FsoFrameworkTestTestCaseAdaptor** _adaptors;
	gint _adaptors_length1;
	gint __adaptors_size_;
};

typedef void (*FsoFrameworkTestTestCaseTestMethod) (void* user_data);
typedef void (*FsoFrameworkTestAsyncBegin) (GAsyncReadyCallback callback, void* callback_target, void* user_data);
typedef void (*FsoFrameworkTestAsyncFinish) (GAsyncResult* _result_, void* user_data, GError** error);
struct _FsoFrameworkTestTestCaseAdaptor {
	GTypeInstance parent_instance;
	volatile int ref_count;
	FsoFrameworkTestTestCaseAdaptorPrivate * priv;
	gboolean is_async;
	FsoFrameworkTestAsyncBegin async_begin;
	gpointer async_begin_target;
	FsoFrameworkTestAsyncFinish async_finish;
	gpointer async_finish_target;
};

struct _FsoFrameworkTestTestCaseAdaptorClass {
	GTypeClass parent_class;
	void (*finalize) (FsoFrameworkTestTestCaseAdaptor *self);
};

struct _FsoFrameworkTestTestCaseAdaptorPrivate {
	gchar* _name;
	gint _async_timeout;
	FsoFrameworkTestTestCaseTestMethod _test;
	gpointer _test_target;
	FsoFrameworkTestTestCase* _test_case;
};

struct _FsoFrameworkTestTestCaseParamSpecAdaptor {
	GParamSpec parent_instance;
};


static gpointer fso_framework_test_test_case_parent_class = NULL;
static gpointer fso_framework_test_test_case_adaptor_parent_class = NULL;

GType fso_framework_test_test_case_get_type (void) G_GNUC_CONST;
static gpointer fso_framework_test_test_case_adaptor_ref (gpointer instance);
static void fso_framework_test_test_case_adaptor_unref (gpointer instance);
static GParamSpec* fso_framework_test_test_case_param_spec_adaptor (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) G_GNUC_UNUSED;
static void fso_framework_test_test_case_value_set_adaptor (GValue* value, gpointer v_object) G_GNUC_UNUSED;
static void fso_framework_test_test_case_value_take_adaptor (GValue* value, gpointer v_object) G_GNUC_UNUSED;
static gpointer fso_framework_test_test_case_value_get_adaptor (const GValue* value) G_GNUC_UNUSED;
static GType fso_framework_test_test_case_adaptor_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define FSO_FRAMEWORK_TEST_TEST_CASE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), FSO_FRAMEWORK_TEST_TYPE_TEST_CASE, FsoFrameworkTestTestCasePrivate))
enum  {
	FSO_FRAMEWORK_TEST_TEST_CASE_DUMMY_PROPERTY
};
FsoFrameworkTestTestCase* fso_framework_test_test_case_construct (GType object_type, const gchar* name);
void fso_framework_test_test_case_add_test (FsoFrameworkTestTestCase* self, const gchar* name, FsoFrameworkTestTestCaseTestMethod test, void* test_target);
static FsoFrameworkTestTestCaseAdaptor* fso_framework_test_test_case_adaptor_new (const gchar* name, FsoFrameworkTestTestCaseTestMethod test, void* test_target, FsoFrameworkTestTestCase* test_case);
static FsoFrameworkTestTestCaseAdaptor* fso_framework_test_test_case_adaptor_construct (GType object_type, const gchar* name, FsoFrameworkTestTestCaseTestMethod test, void* test_target, FsoFrameworkTestTestCase* test_case);
static void _vala_array_add1 (FsoFrameworkTestTestCaseAdaptor*** array, int* length, int* size, FsoFrameworkTestTestCaseAdaptor* value);
static const gchar* fso_framework_test_test_case_adaptor_get_name (FsoFrameworkTestTestCaseAdaptor* self);
static void fso_framework_test_test_case_adaptor_set_up (FsoFrameworkTestTestCaseAdaptor* self, void* fixture);
static void _fso_framework_test_test_case_adaptor_set_up_gtest_func (void* fixture, gpointer self);
static void fso_framework_test_test_case_adaptor_run (FsoFrameworkTestTestCaseAdaptor* self, void* fixture);
static void _fso_framework_test_test_case_adaptor_run_gtest_func (void* fixture, gpointer self);
static void fso_framework_test_test_case_adaptor_tear_down (FsoFrameworkTestTestCaseAdaptor* self, void* fixture);
static void _fso_framework_test_test_case_adaptor_tear_down_gtest_func (void* fixture, gpointer self);
void fso_framework_test_test_case_add_async_test (FsoFrameworkTestTestCase* self, const gchar* name, FsoFrameworkTestAsyncBegin async_begin, void* async_begin_target, FsoFrameworkTestAsyncFinish async_finish, void* async_finish_target, gint timeout);
static void __lambda10_ (FsoFrameworkTestTestCase* self);
static void ___lambda10__fso_framework_test_test_case_test_method (gpointer self);
static void fso_framework_test_test_case_adaptor_set_async_timeout (FsoFrameworkTestTestCaseAdaptor* self, gint value);
static void _vala_array_add2 (FsoFrameworkTestTestCaseAdaptor*** array, int* length, int* size, FsoFrameworkTestTestCaseAdaptor* value);
void fso_framework_test_test_case_set_up (FsoFrameworkTestTestCase* self);
static void fso_framework_test_test_case_real_set_up (FsoFrameworkTestTestCase* self);
void fso_framework_test_test_case_tear_down (FsoFrameworkTestTestCase* self);
static void fso_framework_test_test_case_real_tear_down (FsoFrameworkTestTestCase* self);
GTestSuite* fso_framework_test_test_case_get_suite (FsoFrameworkTestTestCase* self);
#define FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR, FsoFrameworkTestTestCaseAdaptorPrivate))
enum  {
	FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR_DUMMY_PROPERTY
};
static void _fso_framework_test_test_case_adaptor_printerr_func_stack_trace (const gchar* text);
static void __fso_framework_test_test_case_adaptor_printerr_func_stack_trace_gprint_func (const gchar* text);
static void _fso_framework_test_test_case_adaptor_log_func_stack_trace (FsoFrameworkTestTestCaseAdaptor* self, const gchar* log_domain, GLogLevelFlags log_levels, const gchar* message);
static void __fso_framework_test_test_case_adaptor_log_func_stack_trace_glog_func (const gchar* log_domain, GLogLevelFlags log_levels, const gchar* message, gpointer self);
gboolean fso_framework_test_wait_for_async (gint timeout, FsoFrameworkTestAsyncBegin async_function, void* async_function_target, FsoFrameworkTestAsyncFinish async_finish, void* async_finish_target, GError** error);
static gint fso_framework_test_test_case_adaptor_get_async_timeout (FsoFrameworkTestTestCaseAdaptor* self);
static void fso_framework_test_test_case_adaptor_set_name (FsoFrameworkTestTestCaseAdaptor* self, const gchar* value);
static void fso_framework_test_test_case_adaptor_finalize (FsoFrameworkTestTestCaseAdaptor* obj);
static void fso_framework_test_test_case_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


FsoFrameworkTestTestCase* fso_framework_test_test_case_construct (GType object_type, const gchar* name) {
	FsoFrameworkTestTestCase * self = NULL;
	const gchar* _tmp0_;
	GTestSuite* _tmp1_;
	g_return_val_if_fail (name != NULL, NULL);
	self = (FsoFrameworkTestTestCase*) g_object_new (object_type, NULL);
	_tmp0_ = name;
	_tmp1_ = g_test_create_suite (_tmp0_);
	self->priv->_suite = _tmp1_;
	return self;
}


static gpointer _fso_framework_test_test_case_adaptor_ref0 (gpointer self) {
	return self ? fso_framework_test_test_case_adaptor_ref (self) : NULL;
}


static void _vala_array_add1 (FsoFrameworkTestTestCaseAdaptor*** array, int* length, int* size, FsoFrameworkTestTestCaseAdaptor* value) {
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (FsoFrameworkTestTestCaseAdaptor*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}


static void _fso_framework_test_test_case_adaptor_set_up_gtest_func (void* fixture, gpointer self) {
	fso_framework_test_test_case_adaptor_set_up (self, fixture);
}


static void _fso_framework_test_test_case_adaptor_run_gtest_func (void* fixture, gpointer self) {
	fso_framework_test_test_case_adaptor_run (self, fixture);
}


static void _fso_framework_test_test_case_adaptor_tear_down_gtest_func (void* fixture, gpointer self) {
	fso_framework_test_test_case_adaptor_tear_down (self, fixture);
}


void fso_framework_test_test_case_add_test (FsoFrameworkTestTestCase* self, const gchar* name, FsoFrameworkTestTestCaseTestMethod test, void* test_target) {
	const gchar* _tmp0_;
	FsoFrameworkTestTestCaseTestMethod _tmp1_;
	void* _tmp1__target;
	FsoFrameworkTestTestCaseAdaptor* _tmp2_;
	FsoFrameworkTestTestCaseAdaptor* adaptor;
	FsoFrameworkTestTestCaseAdaptor** _tmp3_;
	gint _tmp3__length1;
	FsoFrameworkTestTestCaseAdaptor* _tmp4_;
	GTestSuite* _tmp5_;
	const gchar* _tmp6_;
	const gchar* _tmp7_;
	GTestCase* _tmp8_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (name != NULL);
	_tmp0_ = name;
	_tmp1_ = test;
	_tmp1__target = test_target;
	_tmp2_ = fso_framework_test_test_case_adaptor_new (_tmp0_, _tmp1_, _tmp1__target, self);
	adaptor = _tmp2_;
	_tmp3_ = self->priv->_adaptors;
	_tmp3__length1 = self->priv->_adaptors_length1;
	_tmp4_ = _fso_framework_test_test_case_adaptor_ref0 (adaptor);
	_vala_array_add1 (&self->priv->_adaptors, &self->priv->_adaptors_length1, &self->priv->__adaptors_size_, _tmp4_);
	_tmp5_ = self->priv->_suite;
	_tmp6_ = fso_framework_test_test_case_adaptor_get_name (adaptor);
	_tmp7_ = _tmp6_;
	_tmp8_ = g_test_create_case (_tmp7_, (gsize) sizeof (FsoFrameworkTestTestCaseAdaptor*), adaptor, (void (*) (void)) _fso_framework_test_test_case_adaptor_set_up_gtest_func, (void (*) (void)) _fso_framework_test_test_case_adaptor_run_gtest_func, (void (*) (void)) _fso_framework_test_test_case_adaptor_tear_down_gtest_func);
	g_test_suite_add (_tmp5_, _tmp8_);
	_fso_framework_test_test_case_adaptor_unref0 (adaptor);
}


static void __lambda10_ (FsoFrameworkTestTestCase* self) {
}


static void ___lambda10__fso_framework_test_test_case_test_method (gpointer self) {
	__lambda10_ (self);
}


static void _vala_array_add2 (FsoFrameworkTestTestCaseAdaptor*** array, int* length, int* size, FsoFrameworkTestTestCaseAdaptor* value) {
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (FsoFrameworkTestTestCaseAdaptor*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}


void fso_framework_test_test_case_add_async_test (FsoFrameworkTestTestCase* self, const gchar* name, FsoFrameworkTestAsyncBegin async_begin, void* async_begin_target, FsoFrameworkTestAsyncFinish async_finish, void* async_finish_target, gint timeout) {
	const gchar* _tmp0_;
	FsoFrameworkTestTestCaseAdaptor* _tmp1_;
	FsoFrameworkTestTestCaseAdaptor* adaptor;
	FsoFrameworkTestAsyncBegin _tmp2_;
	void* _tmp2__target;
	FsoFrameworkTestAsyncFinish _tmp3_;
	void* _tmp3__target;
	gint _tmp4_;
	FsoFrameworkTestTestCaseAdaptor** _tmp5_;
	gint _tmp5__length1;
	FsoFrameworkTestTestCaseAdaptor* _tmp6_;
	GTestSuite* _tmp7_;
	const gchar* _tmp8_;
	const gchar* _tmp9_;
	GTestCase* _tmp10_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (name != NULL);
	_tmp0_ = name;
	_tmp1_ = fso_framework_test_test_case_adaptor_new (_tmp0_, ___lambda10__fso_framework_test_test_case_test_method, self, self);
	adaptor = _tmp1_;
	adaptor->is_async = TRUE;
	_tmp2_ = async_begin;
	_tmp2__target = async_begin_target;
	adaptor->async_begin = _tmp2_;
	adaptor->async_begin_target = _tmp2__target;
	_tmp3_ = async_finish;
	_tmp3__target = async_finish_target;
	adaptor->async_finish = _tmp3_;
	adaptor->async_finish_target = _tmp3__target;
	_tmp4_ = timeout;
	fso_framework_test_test_case_adaptor_set_async_timeout (adaptor, _tmp4_);
	_tmp5_ = self->priv->_adaptors;
	_tmp5__length1 = self->priv->_adaptors_length1;
	_tmp6_ = _fso_framework_test_test_case_adaptor_ref0 (adaptor);
	_vala_array_add2 (&self->priv->_adaptors, &self->priv->_adaptors_length1, &self->priv->__adaptors_size_, _tmp6_);
	_tmp7_ = self->priv->_suite;
	_tmp8_ = fso_framework_test_test_case_adaptor_get_name (adaptor);
	_tmp9_ = _tmp8_;
	_tmp10_ = g_test_create_case (_tmp9_, (gsize) sizeof (FsoFrameworkTestTestCaseAdaptor*), adaptor, (void (*) (void)) _fso_framework_test_test_case_adaptor_set_up_gtest_func, (void (*) (void)) _fso_framework_test_test_case_adaptor_run_gtest_func, (void (*) (void)) _fso_framework_test_test_case_adaptor_tear_down_gtest_func);
	g_test_suite_add (_tmp7_, _tmp10_);
	_fso_framework_test_test_case_adaptor_unref0 (adaptor);
}


static void fso_framework_test_test_case_real_set_up (FsoFrameworkTestTestCase* self) {
}


void fso_framework_test_test_case_set_up (FsoFrameworkTestTestCase* self) {
	g_return_if_fail (self != NULL);
	FSO_FRAMEWORK_TEST_TEST_CASE_GET_CLASS (self)->set_up (self);
}


static void fso_framework_test_test_case_real_tear_down (FsoFrameworkTestTestCase* self) {
}


void fso_framework_test_test_case_tear_down (FsoFrameworkTestTestCase* self) {
	g_return_if_fail (self != NULL);
	FSO_FRAMEWORK_TEST_TEST_CASE_GET_CLASS (self)->tear_down (self);
}


GTestSuite* fso_framework_test_test_case_get_suite (FsoFrameworkTestTestCase* self) {
	GTestSuite* result = NULL;
	GTestSuite* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_suite;
	result = _tmp0_;
	return result;
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static FsoFrameworkTestTestCaseAdaptor* fso_framework_test_test_case_adaptor_construct (GType object_type, const gchar* name, FsoFrameworkTestTestCaseTestMethod test, void* test_target, FsoFrameworkTestTestCase* test_case) {
	FsoFrameworkTestTestCaseAdaptor* self = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	FsoFrameworkTestTestCaseTestMethod _tmp2_;
	void* _tmp2__target;
	FsoFrameworkTestTestCase* _tmp3_;
	FsoFrameworkTestTestCase* _tmp4_;
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (test_case != NULL, NULL);
	self = (FsoFrameworkTestTestCaseAdaptor*) g_type_create_instance (object_type);
	_tmp0_ = name;
	_tmp1_ = g_strdup (_tmp0_);
	_g_free0 (self->priv->_name);
	self->priv->_name = _tmp1_;
	_tmp2_ = test;
	_tmp2__target = test_target;
	self->priv->_test = _tmp2_;
	self->priv->_test_target = _tmp2__target;
	_tmp3_ = test_case;
	_tmp4_ = _g_object_ref0 (_tmp3_);
	_g_object_unref0 (self->priv->_test_case);
	self->priv->_test_case = _tmp4_;
	return self;
}


static FsoFrameworkTestTestCaseAdaptor* fso_framework_test_test_case_adaptor_new (const gchar* name, FsoFrameworkTestTestCaseTestMethod test, void* test_target, FsoFrameworkTestTestCase* test_case) {
	return fso_framework_test_test_case_adaptor_construct (FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR, name, test, test_target, test_case);
}


static void __fso_framework_test_test_case_adaptor_printerr_func_stack_trace_gprint_func (const gchar* text) {
	_fso_framework_test_test_case_adaptor_printerr_func_stack_trace (text);
}


static void __fso_framework_test_test_case_adaptor_log_func_stack_trace_glog_func (const gchar* log_domain, GLogLevelFlags log_levels, const gchar* message, gpointer self) {
	_fso_framework_test_test_case_adaptor_log_func_stack_trace (self, log_domain, log_levels, message);
}


static void fso_framework_test_test_case_adaptor_set_up (FsoFrameworkTestTestCaseAdaptor* self, void* fixture) {
	FsoFrameworkTestTestCase* _tmp0_;
	g_return_if_fail (self != NULL);
	g_set_printerr_handler (__fso_framework_test_test_case_adaptor_printerr_func_stack_trace_gprint_func);
	g_log_set_default_handler (__fso_framework_test_test_case_adaptor_log_func_stack_trace_glog_func, self);
	_tmp0_ = self->priv->_test_case;
	fso_framework_test_test_case_set_up (_tmp0_);
}


static void _fso_framework_test_test_case_adaptor_printerr_func_stack_trace (const gchar* text) {
	gboolean _tmp0_ = FALSE;
	const gchar* _tmp1_;
	gboolean _tmp5_;
	FILE* _tmp6_;
	const gchar* _tmp7_;
	_tmp1_ = text;
	if (_tmp1_ == NULL) {
		_tmp0_ = TRUE;
	} else {
		GEqualFunc _tmp2_;
		const gchar* _tmp3_;
		gboolean _tmp4_ = FALSE;
		_tmp2_ = g_str_equal;
		_tmp3_ = text;
		_tmp4_ = _tmp2_ (_tmp3_, "");
		_tmp0_ = _tmp4_;
	}
	_tmp5_ = _tmp0_;
	if (_tmp5_) {
		return;
	}
	_tmp6_ = stderr;
	_tmp7_ = text;
	fprintf (_tmp6_, "%s", _tmp7_);
	g_on_error_stack_trace ("libtool --mode=execute gdb");
}


static void _fso_framework_test_test_case_adaptor_log_func_stack_trace (FsoFrameworkTestTestCaseAdaptor* self, const gchar* log_domain, GLogLevelFlags log_levels, const gchar* message) {
	GLogFunc _tmp0_;
	void* _tmp0__target;
	const gchar* _tmp1_;
	GLogLevelFlags _tmp2_;
	const gchar* _tmp3_;
	GLogLevelFlags _tmp4_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (message != NULL);
	_tmp0_ = g_log_default_handler;
	_tmp0__target = NULL;
	_tmp1_ = log_domain;
	_tmp2_ = log_levels;
	_tmp3_ = message;
	_tmp0_ (_tmp1_, _tmp2_, _tmp3_, _tmp0__target);
	_tmp4_ = log_levels;
	if ((_tmp4_ & ((G_LOG_LEVEL_WARNING | G_LOG_LEVEL_ERROR) | G_LOG_LEVEL_CRITICAL)) != 0) {
		g_on_error_stack_trace ("libtool --mode=execute gdb");
	}
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	result = self;
	return result;
}


static void fso_framework_test_test_case_adaptor_run (FsoFrameworkTestTestCaseAdaptor* self, void* fixture) {
	gboolean _tmp0_;
	GError * _inner_error_ = NULL;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->is_async;
	if (_tmp0_) {
		{
			gint _tmp1_;
			FsoFrameworkTestAsyncBegin _tmp2_;
			void* _tmp2__target;
			FsoFrameworkTestAsyncFinish _tmp3_;
			void* _tmp3__target;
			gboolean _tmp4_ = FALSE;
			gboolean _tmp5_;
			_tmp1_ = self->priv->_async_timeout;
			_tmp2_ = self->async_begin;
			_tmp2__target = self->async_begin_target;
			_tmp3_ = self->async_finish;
			_tmp3__target = self->async_finish_target;
			_tmp4_ = fso_framework_test_wait_for_async (_tmp1_, _tmp2_, _tmp2__target, _tmp3_, _tmp3__target, &_inner_error_);
			_tmp5_ = _tmp4_;
			if (_inner_error_ != NULL) {
				goto __catch1_g_error;
			}
			g_assert (_tmp5_);
		}
		goto __finally1;
		__catch1_g_error:
		{
			GError* err = NULL;
			GError* _tmp6_;
			const gchar* _tmp7_;
			const gchar* _tmp8_ = NULL;
			gchar* _tmp9_ = NULL;
			gchar* _tmp10_;
			err = _inner_error_;
			_inner_error_ = NULL;
			_tmp6_ = err;
			_tmp7_ = _tmp6_->message;
			_tmp8_ = string_to_string (_tmp7_);
			_tmp9_ = g_strconcat ("Got exception while excuting asynchronous test: ", _tmp8_, NULL);
			_tmp10_ = _tmp9_;
			g_message ("testcase.vala:128: %s", _tmp10_);
			_g_free0 (_tmp10_);
			g_test_fail ();
			_g_error_free0 (err);
		}
		__finally1:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
			g_clear_error (&_inner_error_);
			return;
		}
	} else {
		FsoFrameworkTestTestCaseTestMethod _tmp11_;
		void* _tmp11__target;
		_tmp11_ = self->priv->_test;
		_tmp11__target = self->priv->_test_target;
		_tmp11_ (_tmp11__target);
	}
}


static void fso_framework_test_test_case_adaptor_tear_down (FsoFrameworkTestTestCaseAdaptor* self, void* fixture) {
	FsoFrameworkTestTestCase* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->_test_case;
	fso_framework_test_test_case_tear_down (_tmp0_);
}


static const gchar* fso_framework_test_test_case_adaptor_get_name (FsoFrameworkTestTestCaseAdaptor* self) {
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_name;
	result = _tmp0_;
	return result;
}


static void fso_framework_test_test_case_adaptor_set_name (FsoFrameworkTestTestCaseAdaptor* self, const gchar* value) {
	const gchar* _tmp0_;
	gchar* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	_tmp1_ = g_strdup (_tmp0_);
	_g_free0 (self->priv->_name);
	self->priv->_name = _tmp1_;
}


static gint fso_framework_test_test_case_adaptor_get_async_timeout (FsoFrameworkTestTestCaseAdaptor* self) {
	gint result;
	gint _tmp0_;
	g_return_val_if_fail (self != NULL, 0);
	_tmp0_ = self->priv->_async_timeout;
	result = _tmp0_;
	return result;
}


static void fso_framework_test_test_case_adaptor_set_async_timeout (FsoFrameworkTestTestCaseAdaptor* self, gint value) {
	gint _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	self->priv->_async_timeout = _tmp0_;
}


static void fso_framework_test_test_case_value_adaptor_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void fso_framework_test_test_case_value_adaptor_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		fso_framework_test_test_case_adaptor_unref (value->data[0].v_pointer);
	}
}


static void fso_framework_test_test_case_value_adaptor_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = fso_framework_test_test_case_adaptor_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer fso_framework_test_test_case_value_adaptor_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* fso_framework_test_test_case_value_adaptor_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		FsoFrameworkTestTestCaseAdaptor* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = fso_framework_test_test_case_adaptor_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* fso_framework_test_test_case_value_adaptor_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	FsoFrameworkTestTestCaseAdaptor** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = fso_framework_test_test_case_adaptor_ref (value->data[0].v_pointer);
	}
	return NULL;
}


static GParamSpec* fso_framework_test_test_case_param_spec_adaptor (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	FsoFrameworkTestTestCaseParamSpecAdaptor* spec;
	g_return_val_if_fail (g_type_is_a (object_type, FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


static gpointer fso_framework_test_test_case_value_get_adaptor (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR), NULL);
	return value->data[0].v_pointer;
}


static void fso_framework_test_test_case_value_set_adaptor (GValue* value, gpointer v_object) {
	FsoFrameworkTestTestCaseAdaptor* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		fso_framework_test_test_case_adaptor_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		fso_framework_test_test_case_adaptor_unref (old);
	}
}


static void fso_framework_test_test_case_value_take_adaptor (GValue* value, gpointer v_object) {
	FsoFrameworkTestTestCaseAdaptor* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, FSO_FRAMEWORK_TEST_TEST_CASE_TYPE_ADAPTOR));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		fso_framework_test_test_case_adaptor_unref (old);
	}
}


static void fso_framework_test_test_case_adaptor_class_init (FsoFrameworkTestTestCaseAdaptorClass * klass) {
	fso_framework_test_test_case_adaptor_parent_class = g_type_class_peek_parent (klass);
	FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR_CLASS (klass)->finalize = fso_framework_test_test_case_adaptor_finalize;
	g_type_class_add_private (klass, sizeof (FsoFrameworkTestTestCaseAdaptorPrivate));
}


static void fso_framework_test_test_case_adaptor_instance_init (FsoFrameworkTestTestCaseAdaptor * self) {
	self->priv = FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR_GET_PRIVATE (self);
	self->is_async = FALSE;
	self->ref_count = 1;
}


static void fso_framework_test_test_case_adaptor_finalize (FsoFrameworkTestTestCaseAdaptor* obj) {
	FsoFrameworkTestTestCaseAdaptor * self;
	self = FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR (obj);
	_g_free0 (self->priv->_name);
	_g_object_unref0 (self->priv->_test_case);
}


static GType fso_framework_test_test_case_adaptor_get_type (void) {
	static volatile gsize fso_framework_test_test_case_adaptor_type_id__volatile = 0;
	if (g_once_init_enter (&fso_framework_test_test_case_adaptor_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { fso_framework_test_test_case_value_adaptor_init, fso_framework_test_test_case_value_adaptor_free_value, fso_framework_test_test_case_value_adaptor_copy_value, fso_framework_test_test_case_value_adaptor_peek_pointer, "p", fso_framework_test_test_case_value_adaptor_collect_value, "p", fso_framework_test_test_case_value_adaptor_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (FsoFrameworkTestTestCaseAdaptorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) fso_framework_test_test_case_adaptor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FsoFrameworkTestTestCaseAdaptor), 0, (GInstanceInitFunc) fso_framework_test_test_case_adaptor_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType fso_framework_test_test_case_adaptor_type_id;
		fso_framework_test_test_case_adaptor_type_id = g_type_register_fundamental (g_type_fundamental_next (), "FsoFrameworkTestTestCaseAdaptor", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&fso_framework_test_test_case_adaptor_type_id__volatile, fso_framework_test_test_case_adaptor_type_id);
	}
	return fso_framework_test_test_case_adaptor_type_id__volatile;
}


static gpointer fso_framework_test_test_case_adaptor_ref (gpointer instance) {
	FsoFrameworkTestTestCaseAdaptor* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


static void fso_framework_test_test_case_adaptor_unref (gpointer instance) {
	FsoFrameworkTestTestCaseAdaptor* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		FSO_FRAMEWORK_TEST_TEST_CASE_ADAPTOR_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void fso_framework_test_test_case_class_init (FsoFrameworkTestTestCaseClass * klass) {
	fso_framework_test_test_case_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (FsoFrameworkTestTestCasePrivate));
	FSO_FRAMEWORK_TEST_TEST_CASE_CLASS (klass)->set_up = fso_framework_test_test_case_real_set_up;
	FSO_FRAMEWORK_TEST_TEST_CASE_CLASS (klass)->tear_down = fso_framework_test_test_case_real_tear_down;
	G_OBJECT_CLASS (klass)->finalize = fso_framework_test_test_case_finalize;
}


static void fso_framework_test_test_case_instance_init (FsoFrameworkTestTestCase * self) {
	FsoFrameworkTestTestCaseAdaptor** _tmp0_ = NULL;
	self->priv = FSO_FRAMEWORK_TEST_TEST_CASE_GET_PRIVATE (self);
	_tmp0_ = g_new0 (FsoFrameworkTestTestCaseAdaptor*, 0 + 1);
	self->priv->_adaptors = _tmp0_;
	self->priv->_adaptors_length1 = 0;
	self->priv->__adaptors_size_ = self->priv->_adaptors_length1;
}


static void fso_framework_test_test_case_finalize (GObject* obj) {
	FsoFrameworkTestTestCase * self;
	self = FSO_FRAMEWORK_TEST_TEST_CASE (obj);
	self->priv->_adaptors = (_vala_array_free (self->priv->_adaptors, self->priv->_adaptors_length1, (GDestroyNotify) fso_framework_test_test_case_adaptor_unref), NULL);
	G_OBJECT_CLASS (fso_framework_test_test_case_parent_class)->finalize (obj);
}


GType fso_framework_test_test_case_get_type (void) {
	static volatile gsize fso_framework_test_test_case_type_id__volatile = 0;
	if (g_once_init_enter (&fso_framework_test_test_case_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (FsoFrameworkTestTestCaseClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) fso_framework_test_test_case_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FsoFrameworkTestTestCase), 0, (GInstanceInitFunc) fso_framework_test_test_case_instance_init, NULL };
		GType fso_framework_test_test_case_type_id;
		fso_framework_test_test_case_type_id = g_type_register_static (G_TYPE_OBJECT, "FsoFrameworkTestTestCase", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_once_init_leave (&fso_framework_test_test_case_type_id__volatile, fso_framework_test_test_case_type_id);
	}
	return fso_framework_test_test_case_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



