/* testnetlink.c generated by valac 0.16.1, the Vala compiler
 * generated from testnetlink.vala, do not modify */


#include <glib.h>
#include <glib-object.h>
#include <fsobasics.h>
#include <stdlib.h>
#include <string.h>

#define _g_free0(var) (var = (g_free (var), NULL))



void test_netlink_getaddress (void);
void _vala_main (gchar** args, int args_length1);


/**
 * Copyright (C) 2009-2012 Michael 'Mickey' Lauer <mlauer@vanille-media.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */
void test_netlink_getaddress (void) {
	gchar* _tmp0_ = NULL;
	gchar* _tmp1_;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_;
	_tmp0_ = fso_framework_net_ipv4AddressForInterface ("lo");
	_tmp1_ = _tmp0_;
	g_assert (g_strcmp0 (_tmp1_, "127.0.0.1") == 0);
	_g_free0 (_tmp1_);
	_tmp2_ = fso_framework_net_ipv4AddressForInterface ("abc7");
	_tmp3_ = _tmp2_;
	g_assert (g_strcmp0 (_tmp3_, "unknown") == 0);
	_g_free0 (_tmp3_);
}


void _vala_main (gchar** args, int args_length1) {
	g_test_init (&args_length1, &args, NULL);
	g_test_run ();
}


int main (int argc, char ** argv) {
	g_thread_init (NULL);
	g_type_init ();
	_vala_main (argv, argc);
	return 0;
}



